@@ -1,3 +1,11 @@
+"""
+It is a very similar class to that of Processor, but it contains the con-
+trol voltages, declared as learnable parameters. Therefore, it only has the same
+input dimensions as the number of available data input electrodes. It is also a
+child of torch.nn.Module, and it allows for representing a layer of DNPUs in a time-
+multiplexing fashion (with the same Processor instance). It also enables applying
+linear transformations to the inputs before passing them to the processor.
+"""
 from random import random
 import torch
 import collections
@@ -30,7 +38,7 @@ class DNPU(nn.Module):
         Initialises the super class and makes a separation between those electrodes that are data
         input and those that are control.
 
-        Attributes
+        Parameters
         ----------
         processor : Processor
             An instance of a Processor class representing either a simulation processor or a
@@ -76,7 +84,7 @@ class DNPU(nn.Module):
         """
         Sets the type of forward pass that is going to be used.
 
-        Attributes
+        Parameters
         ----------
         forward_pass_type : str
             It indicates if the forward pass for more than one DNPU devices on time-multiplexing
@@ -100,7 +108,7 @@ class DNPU(nn.Module):
         It counts how many control and data input electrodes are
         going to be declared using the data_input_indices variable.
 
-        Attributes
+        Parameters
         ----------
         data_input_indices: Sequence[int]
             Specifies which electrodes are going to be used for inputing data. The reminder of the
@@ -508,7 +516,8 @@ class DNPU(nn.Module):
         [Batch_size, window_no, in_chanels, node_no, input_electrode_no]. This function
         has to be called from outside the module, after its initialisation.
 
-        Attributes:
+        Parameters
+        ----------
         input_range : list
             The range that the original raw input data is going to have. It can be specified with
             two values [min, max], representing the minimum and maximum values that the input data
@@ -520,6 +529,7 @@ class DNPU(nn.Module):
         strict : boolean
             Defines if the input is going to be clipped before doing the linear transformation in
             order to ensure that the transformation is correct.
+
         """
         self.input_transform = True
         self.input_clip = strict
